<?php

$dir = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR;
$files = array(
    'chunk.php', 'archiveHelper.php', 'defines.php', 'editorHelper.php', 'filesHelper.php', 'logFormatter.php', 'previewHelper.php', 'processThemeHelper.php', 'templateBuilder.php', 'themeManagement.php', 'themlerMethods.php', 'version.php'
);
foreach ($files as $file)
    require_once($dir . $file);

set_error_handler('permissionErrorHandler', E_ALL);

function themeler_get_actions() {
   return array('process_request', 'update', 'export', 'save', 'remove', 'copy', 'rename', 'canrename', 'get_files', 'set_files', 'download', 'clear', 'activate', 'validation', 'upload_image', 'upload_theme', 'refresh_templates', 'refresh_themes', 'fso_zip', 'unzip');
}

function get_urls($base_url, $theme_name) {
    $templates = get_templates_url($base_url, $theme_name);
    $clean_url = get_base_clean_urls($base_url);
    $path = $clean_url . themler_get_base_path() . '/';
    $result = array('admin' => $clean_url . 'admin', 'themler' => $path);
    if (isset($templates['home'])) $result['preview'] = $templates['home'];
    $actions = themeler_get_actions();
    foreach ($actions as $action)
        $result[$action] = $path . ((strpos($action, 'upload') !== false) || (strpos($action, 'unzip') !== false) ?  'upload&theme=' . $theme_name . '&callback=' . $action : $action);

    return $result;
}

function themler_process_request() {
    themler_login_user();
    $action = getValue('action', '');
    $method = 'themler_'.$action;
    if (function_exists($method)) {
        try {
            $method();
        } catch (PermissionsException $e) {
            $response = get_permission_response($e);
            themler_output($response);
        }
    }
}

function themler_menu() {
    $actions = themeler_get_actions();
    $base_path = themler_get_base_path();
    $items = array();
    $items[$base_path] = array(
        'access callback' => TRUE,
        'page callback' => 'themler_page',
        'access arguments' => array('administer site configuration'),
        'type' => MENU_CALLBACK
    );

    foreach ($actions as $action) {
        $method = strpos($action, 'upload') !== false || strpos($action, 'unzip') !== false ? 'upload' : $action;
        $items[$base_path . '/' . $method] = array(
            'access callback' => TRUE,
            'page callback' => 'themler_' . $method,
            'access arguments' => array('administer site configuration'),
            'type' => MENU_CALLBACK
        );
    }
    return $items;
}

/**
 * Updates Themler theme
 */
function update($theme_name) {
    ProviderLog::start('update');
    $theme_dir = get_theme_dir($theme_name);
    $preview_theme_dir = get_preview_theme_dir($theme_dir);
    $preview_theme_name = get_preview_theme_name($theme_name);
    $replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => $theme_name, 'new_name' => $preview_theme_name);
    $helper = new PreviewHelper($theme_dir);
    $callback = array($helper, 'restoreDataId');
    ProviderLog::start('Update files');
    $changedFiles = get_preview_changed_files($theme_dir);
    if ($changedFiles === null || !file_exists($preview_theme_dir)) {
        FilesHelper::copy_recursive($theme_dir, $preview_theme_dir, $callback, $replace_data);
        $dirs = array('export', 'project');
        foreach ($dirs as $dir)
            FilesHelper::empty_dir($preview_theme_dir . '/' . $dir, true);
    } else {
        // copy only changed files
        foreach($changedFiles as $file) {
            if (file_exists($theme_dir . $file)) {
                FilesHelper::copy_recursive($theme_dir . $file, $preview_theme_dir . $file, $callback, $replace_data);
            }
            // $changed_files contains {$preview_name}.inf not {$theme_name}.inf - so we must rename it before
            elseif (strpos($file, $preview_theme_name . INF_EXT) !== false) {
                $new_file = str_replace($preview_theme_name . INF_EXT, $theme_name . INF_EXT, $file);
                if (file_exists($theme_dir . $new_file)) {
                    FilesHelper::copy_recursive($theme_dir . $new_file, $preview_theme_dir . $file, $callback, $replace_data);
                }
            }
            else {
                if (is_file($preview_theme_dir . $file)) {
                    FilesHelper::delete_file($preview_theme_dir . $file);
                }
            }
        }
        set_preview_changed_files($theme_dir, $preview_theme_dir, array());
    }
    ProviderLog::end('Update files');
    ProviderLog::end('update');

    return array('result' => 'done', 'log' => ProviderLog::getLog());
}

/**
 * Exports Themler theme
 */
function export($theme_name, $info) {
    $chunk = new Chunk();
    if (!$chunk->save($info)) {
        header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
        return array();
    }

    if ($chunk->last()) {
        ProviderLog::start('export');
        ProviderLog::start('export:json_decode');
        $content = json_decode($chunk->complete(), true);
        ProviderLog::end('export:json_decode');
        process_theme($theme_name, $content);
        ProviderLog::end('export');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }
    return array('result' => 'processed');
}

/**
 * Saves Themler project to current theme
 */
function save($info, $theme_name, $isNew) {
    $modules_dir = get_modules_dir();
    $theme_dir = get_theme_dir($theme_name);
    $cms_version = get_cms_version();
    $chunk = new Chunk();
    if (!$chunk->save($info)) {
        header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
        return array();
    }

    if ($chunk->last()) {
        ProviderLog::start('save');
        ProviderLog::start('save:json decode');
        $result = json_decode($chunk->complete(), true);
        ProviderLog::end('save:json decode');
        $preview_theme_dir = get_preview_theme_dir($theme_dir);
        $preview_theme_name = get_preview_theme_name($theme_name);
        $replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => $preview_theme_name, 'new_name' => $theme_name);

        $screenshots    = array_key_exists('thumbnails', $result) > 0 ? $result['thumbnails'] : '';
        $css_js_sources = array_key_exists('cssJsSources', $result) > 0 ? $result['cssJsSources'] : '';
        $md5_hashes     = array_key_exists('md5Hashes', $result) > 0 ? $result['md5Hashes'] : '';
        $project_data   = array_key_exists('projectData', $result) > 0 ? $result['projectData'] : '';
        $images = empty($result['images']) ? null : $result['images'];

        $helper = new PreviewHelper($theme_dir);
        $callback = array($helper, 'removeDataId');
        $changed_files = get_preview_changed_files($theme_dir);

        ProviderLog::start('Save files');
        FilesHelper::delete_file($theme_dir . '/css/style.min.css');
        FilesHelper::delete_file($theme_dir . '/css/bootstrap.min.css');

        foreach($changed_files as $file) {
            if (file_exists($preview_theme_dir . $file)) {
                $theme_file_dir = dirname($preview_theme_dir . $file);
                // current cms version or common themler files
                if ((strpos($file, 'themler/') !== false) && (has_current_version($file) || !has_version($file))) {
                    // replace current version in path if it's available there
                    $new_file = FilesHelper::replace_content($file, '/versions/' . $cms_version .'/', '/');
                    $new_file = FilesHelper::replace_content($new_file, INF_EXT, INFO_EXT);
                    FilesHelper::copy_recursive($preview_theme_dir . $file, $modules_dir . $new_file);
                }
                if (preg_match('#[\\\/]images$#', $theme_file_dir)) { // file inside 'images' directory moves to base theme
                    // TODO: add permissions process
                    FilesHelper::rename_file($preview_theme_dir . $file, $theme_dir . $file);
                } else {
                    FilesHelper::copy_recursive($preview_theme_dir . $file, $theme_dir . $file, $callback, $replace_data);
                }
            } else {
                $helper->removeKey($file);
                FilesHelper::delete_file($theme_dir . $file);
            }
        }
        ProviderLog::end('Save files');
        set_preview_changed_files($theme_dir, $preview_theme_dir, array());
        $helper->save();

        ProviderLog::start('Save images');
        process_images($theme_dir, $images, $changed_files);
        FilesHelper::empty_dir($preview_theme_dir . '/images');
        ProviderLog::end('Save images');

        save_screenshots(array($theme_dir, $preview_theme_dir), $screenshots);

        if (!empty($project_data))
            save_project($project_data, $theme_dir);

        if (!empty($css_js_sources ))
            set_theme_cache($theme_dir, $css_js_sources);
        
        if (!empty($md5_hashes))
            set_theme_hashes($theme_dir, $md5_hashes);

        ProviderLog::start('drupal_flush_all_caches');
        drupal_flush_all_caches();
        ProviderLog::end('drupal_flush_all_caches');

        ProviderLog::end('save');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }
    return array('result' => 'processed');
}

function get_files($theme_name, $mask, $filter) {
    $theme_dir = get_theme_dir($theme_name);
    $files     = _getFiles($theme_dir . '/css/{' . $mask . '}', GLOB_BRACE);

    $out_files = array();
    foreach ($files as $file) {
        $info = pathinfo($file);
        $filename = $info['basename'];
        if (is_dir($file) || !preg_match("#bootstrap\.css|style\.css#", $filename) || ($filter && preg_match("#$filter#", $filename))) {
            continue;
        }
        $out_files[$filename] = FilesHelper::read_file($file);
    }

    return array('result' => 'done', 'files' => $out_files);
}

function set_files() {
    $files = array();
    $response = null;

    $data = isset($_POST['files']) ? $_POST['files'] : false;
    if ($data) {
        $files = json_decode($data, true);
        if (null === $files) {
            trigger_error('Data is incorrect in setFiles method: "' . $data . '"', E_USER_ERROR);
        }
        $response = array('result' => 'OK');
    }
    else
    {
        $chunk = new Chunk();

        if (!$chunk->save(getChunkInfo())) {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            return array();
        }

        if ($chunk->last())
        {
            $files = json_decode($chunk->complete(), true);
            $response = array('result' => 'done');
        }
        else 
        {
            $response = array('result' => 'processed');
        }
    }

    if ($files && count($files)) {
        _setFiles($files);
    }

    return $response;
}

function fso_zip($theme_name) {
    $chunk = new Chunk();

    if (!$chunk->save(getChunkInfo())) {
        header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
        return array();
    }

    if (!$chunk->last()) {
        return array('result' => 'processed');
    }
    
    $data = json_decode($chunk->complete(), true);
    if (!isset($data['fso']))
        throw new Exception('Empty fso');

    $base_upload_dir = get_themler_storage_dir();
    $tmp_dir = $base_upload_dir . '/zip-data.tmp';
    $zip_file = $base_upload_dir . '/zip-data.zip';
    FilesHelper::empty_dir($tmp_dir);

    $changed_files = array();
    $replace_data = array();
    $preview_theme_name = $theme_name . PREVIEW_SUFFIX;

    fill_theme_storage($data['fso'], $changed_files, $replaceInfo, false, $tmp_dir, $preview_theme_name, $tmp_dir, true);

    $archive = new PclZip($zip_file);

    if (0 == $archive->create($tmp_dir, PCLZIP_OPT_REMOVE_PATH, $tmp_dir))
        throw new Exception("Extract error : " . $archive->errorInfo(true));

    $response = array('result' => 'done', 'data' => base64_encode(FilesHelper::read_file($zip_file)));
    FilesHelper::empty_dir($tmp_dir);
    FilesHelper::delete_file($zip_file);
    return $response;
}

function _getFiles($mask, $flags) {
    $files = glob($mask, $flags);
    if (!is_array($files)) {
        $files = array();
    }
    $folders = glob(dirname($mask) . '/*', GLOB_ONLYDIR | GLOB_NOSORT);
    if (!is_array($folders)) {
        $folders = array();
    }
    foreach ($folders as $dir) {
        $files = array_merge($files, _getFiles($dir . '/' . basename($mask), $flags));
    }

    return $files;
}

function _setFiles($files) {
    $theme_name = $_POST['themeName'];
    $theme_dir = get_theme_dir($theme_name);
    foreach ($files as $filename => $content) {
        $fileExt = pathinfo($filename, PATHINFO_EXTENSION);
        if (in_array($fileExt, array('css'))) {
            FilesHelper::write_file($theme_dir . '/css/' . $filename, $content, LOCK_EX);
        } elseif ($filename === '/export/themler.manifest') {
            theme_save_manifest($theme_name, $content);
        } else {
            FilesHelper::write_file($theme_dir . '/' . $filename, $content, LOCK_EX);
        }
    }

    return;
}

function clear_chunk() {
    $chunk = new Chunk();
    FilesHelper::remove_dir($chunk->UPLOAD_PATH);
}

function getChunkInfo() {
    return array(
        'id' => isset($_POST['id']) ? $_POST['id'] : '',
        'content' => isset($_POST['content']) ? $_POST['content'] : '',
        'current' => isset($_POST['current']) ? $_POST['current'] : '',
        'total' => isset($_POST['total']) ? $_POST['total'] : '',
        'encode' => !empty($_POST['encode']),
        'blob' => !empty($_POST['blob'])
    );
}

function get_base_clean_urls($base_url) {
    // always use q param in Themler
    return $base_url . '?q=';
}

function has_clean_urls() {
    // TODO: use this method in future. Drupal .htaccess file is more important than apache mod_rewrite
    if (function_exists('apache_get_modules')) {
        $modules = apache_get_modules();
        $mod_rewrite = in_array('mod_rewrite', $modules);
    } else {
        $mod_rewrite = getenv('HTTP_MOD_REWRITE') == 'On' ? true : false ;
    }

    global $conf;
    return isset($conf) && isset($conf['clean_url']) && $conf['clean_url'] && $mod_rewrite;
}

function check_permissions($theme_name) {
    $theme_dir = get_theme_dir($theme_name);
    $dirs = get_premission_dirs();
    $dirs[] = $theme_dir;
    $dirs[] = get_preview_theme_dir($theme_dir);

    foreach ($dirs as $dir) {
        $rootOnly = strpos($dir, '/themes/') === FALSE; // check themes subdirectories only
        $check = FilesHelper::check_files($dir, true, true, $rootOnly);
    }
}

function get_permission_response($e) {
    return array('result' => 'fail', 'type' => 'permission', 'error' => $e->getMessage());
}

restore_error_handler();