<?php

require_once('template.php');

function Untitled_settings($saved_settings) { // 6.x
    $form = array();
    Untitled_get_form_settings($form);
    return $form;
}

function Untitled_form_system_theme_settings_alter(&$form, &$form_state) { // 7.x
    Untitled_get_form_settings($form);
}

function Untitled_get_form_settings(&$form) {
    if (!file_exists(dirname(__FILE__).'/project/project.json'))
        return; // base theme only

    $templates = Untitled_get_templates();
    $version = Untitled_get_major_version();

    $form['templates']  = array(
        '#type' => 'fieldset',
        '#weight' => '-3',
        '#title' => t('Template settings')
    );
    $form['templates']['settings'] = Untitled_render_templates_options($templates, $version);
    $form['templates']['save_templates'] = array(
        '#type' => 'submit',
        '#value' => t('Save')
    );
    $form['templates']['save_templates']['#submit'][] = 'Untitled_save_templates_submit';

    $custom_templates = Untitled_render_custom_templates_form($templates, $version);
    if (!empty($custom_templates)) {
        $form['customtemplates'] =  array(
            '#type' => 'fieldset',
            '#weight' => '-2',
            '#title' => t('Custom page template URLs')
        );
        $form['customtemplates']['settings'] = $custom_templates;
        $form['customtemplates']['clear_templates'] = array(
            '#type' => 'submit',
            '#value' => t('Clear')
        );
        $form['customtemplates']['clear_templates']['#submit'][] = 'Untitled_clear_templates_submit';
    }

    if (function_exists('themler_show')) { // Installed Themler only
        $form['themler']  = array(
            '#type' => 'fieldset',
            '#weight' => '-1',
            '#title' => t('Themler')
        );
          
        $form['themler']['edit_theme'] = array(
            '#type' => 'submit',
            '#value' => t('Edit theme')
        );
        $form['themler']['edit_theme']['#submit'][] = 'Untitled_edit_theme_submit';
    }
}

function Untitled_edit_theme_submit($form, &$form_state) {
    $path = themler_get_base_path();
    $theme_name = 'Untitled';
    $version = theme_get_manifest_version($theme_name);
    $query = array('query' => array('theme' => $theme_name, 'ver' => $version));
    if (Untitled_EXPORT_VERSION < 7) { // drupal 6.x
        $query = array('theme' => $theme_name, 'ver' => $version);
    }
    $form_state['redirect'] = array($path, $query);
}

function Untitled_get_templates() {
    $path = dirname(__FILE__) . '/templates/templates.php';
    if (file_exists($path)) {
        require($path);
        return $templates;
    }
    return array();
}

function Untitled_render_templates_options($templates, $version) {
    $result = array();
    foreach ($templates as $type => $template) {
        $options = array();
        if (isset($templates[$type])) {
            $title = '';
            foreach ($template as $t) {
                $options[$t['d'.$version]] = $t['caption'];
                $title = $t['ui_title'];
            }

            $defaultValue = variable_get(Untitled_get_template_key($type), false);
            if (!$defaultValue) { // backward: template type might not be set yet
                $defaultValue = Untitled_get_backward_template($type);
            }

            $result[$type] = array(
                '#type' => 'select',
                '#options' => $options,
                '#title' => $title,
                //'#description' => t('Choose "'.$type.'" to always use on the '.$type.' page.'),
                '#default_value' => $defaultValue,
            );
            if ($type === 'customTemplate') {
                $result[Untitled_get_custom_template_url_key()] = array(
                    '#type' => 'textfield',
                    '#title' => t('Custom page template URL'),
                    '#description' => t('Input page URL you wish to use custom template.'),
                    '#default_value' => '',
                    '#field_prefix' => url(NULL, array('absolute' => TRUE)) . (variable_get('clean_url', 0) ? '' : '?q='), // TODO: 6.x
                ); 
            }
        }
    }

    return $result;
}

function Untitled_render_custom_templates_form($templates, $version) {
    $result = array();
    $name = Untitled_get_custom_template_urls_list_key();
    $allCustomUrls = variable_get($name, '');
    if ($allCustomUrls) { // build list of all custom urls
        $explode = explode(',', $allCustomUrls);
        $unique  = array_unique($explode);
        $options = array();
        foreach ($unique as $u) {
            $url = variable_get($u.'_url', false);
            $t = variable_get($u, false);
            
            $customTemplateVarName = Untitled_get_custom_template_variable_name($url);
            $customTemplateName = variable_get($customTemplateVarName, false);
            $caption = '';
            foreach ($templates['customTemplate'] as $customTemplate) {
                if (isset($customTemplate['d'.$version]) && ($customTemplateName == $customTemplate['d'.$version])) {
                    $caption = $customTemplate['caption'];
                }
            }

            $options[$caption][$u] = $url;
        }
    
        if (!empty($options)) {
            $result[$name] = array(
                '#type' => 'select',
                '#options' => $options,
                '#description' => t('List of page URLs using custom templates.'),
                '#default_value' => '',                    
            );
        }
    }
    
    return $result;
}

function Untitled_save_templates_submit($form, &$form_state) {
    $customTemplateUrlKey = Untitled_get_custom_template_url_key();
    $templates = Untitled_get_templates();
    foreach ($templates as $type => $template) {
        if (array_key_exists($type, $form_state['values'])) {
            $key = Untitled_get_template_key($type);
            variable_set($key, $form_state['values'][$type]);
        }
    }
    if (array_key_exists($customTemplateUrlKey, $form_state['values'])) {
        $value = $form_state['values'][$customTemplateUrlKey];
        if (!empty($value)) {
            $source = drupal_get_normal_path($value);
            if ((function_exists('drupal_valid_path') && !drupal_valid_path($source)) ||
                (function_exists('menu_valid_path') && !menu_valid_path(array('link_path' => $path)))) {
                form_set_error('source', t("The path '@link_path' is either invalid or you do not have access to it.", array('@link_path' => $source)));
            } else {
                $key = Untitled_get_custom_template_variable_name($value);
                variable_set($key, $form_state['values']['customTemplate']);
                variable_set($key.'_url', $value);

                Untitled_save_custom_template_urls_list($key);
            }
        }
    }
    //drupal_flush_all_caches();
    drupal_set_message('Templates configuration has been saved');
}

function Untitled_clear_templates_submit($form, &$form_state) {
    $name = Untitled_get_custom_template_urls_list_key();
    if (array_key_exists($name, $form_state['values'])) {
        $value = $form_state['values'][$name];
        variable_del($value); 
        variable_del($value.'_url');
        
        $allCustomUrls = variable_get($name, '');
        $explode = explode(',', $allCustomUrls);
        $unique  = array_unique($explode);
        if (($key = array_search($value, $unique)) !== false) {
            unset($unique[$key]);
        }
        $implode = implode(',', $unique);
        variable_set($name, $implode);
    }
}

function Untitled_save_custom_template_urls_list($urlKey) {
    // save all custom template urls
    $name = Untitled_get_custom_template_urls_list_key();
    $allCustomUrls = variable_get($name, '');
    $allCustomUrls .= ($allCustomUrls ? ',' : '') . $urlKey;
    
    $explode = explode(',', $allCustomUrls);
    $unique  = array_unique($explode);
    $implode = implode(',', $unique);

    variable_set($name, $implode);
}